searchData={"items":[{"type":"module","title":"Aurora.Ctx.Core","doc":"Core implementation of common database operations using Ecto.\n\nProvides functions for:\n- CRUD operations (create, read, update, delete)\n- Pagination (list_paginated, navigation)\n- Query operations (count, list)\n- Record management (change, new)","ref":"Aurora.Ctx.Core.html"},{"type":"function","title":"Aurora.Ctx.Core.change/2","doc":"Creates a changeset for the given entity.\n\nParameters:\n- entity (Ecto.Schema.t()) - Entity for changeset\n- attrs (map) - Changeset attributes\n\nReturns Ecto.Changeset.","ref":"Aurora.Ctx.Core.html#change/2"},{"type":"function","title":"Aurora.Ctx.Core.change/3","doc":"Creates a changeset for the given entity using a specific changeset function.","ref":"Aurora.Ctx.Core.html#change/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.change/3","doc":"- entity (Ecto.Schema.t() | Ecto.Changeset.t()) - Existing record or changeset\n- changeset_function (atom()) - Custom changeset function\n- attrs (map()) - Changeset attributes","ref":"Aurora.Ctx.Core.html#change/3-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.change/3","doc":"- Ecto.Changeset.t()","ref":"Aurora.Ctx.Core.html#change/3-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.change/3","doc":"alias MyApp.Product\n    product = %Product{name: \"Widget\"}\n    Core.change(product, %{name: \"New Widget\"})\n    #=> #Ecto.Changeset \n\n    Core.change(product, :custom_changeset, %{status: \"active\"})\n    #=> #Ecto.Changeset","ref":"Aurora.Ctx.Core.html#change/3-examples"},{"type":"function","title":"Aurora.Ctx.Core.count/3","doc":"Counts total records matching query conditions.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to query\n- opts (keyword) - Optional query filtering options\n\nReturns total count of matching records.","ref":"Aurora.Ctx.Core.html#count/3"},{"type":"function","title":"Aurora.Ctx.Core.create/3","doc":"Creates a new record.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to create\n- attrs (map) - Attributes for the new record\n\nReturns {:ok, schema} on success, {:error, changeset} on failure.","ref":"Aurora.Ctx.Core.html#create/3"},{"type":"function","title":"Aurora.Ctx.Core.create/4","doc":"Creates a new record with optional custom changeset function.","ref":"Aurora.Ctx.Core.html#create/4"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.create/4","doc":"- repo_module (module()) - Ecto.Repo module to use\n- schema_module (module()) - Schema module for the record\n- changeset_function (atom()) - Custom changeset function to use\n- attrs (map()) - Attributes for the new record","ref":"Aurora.Ctx.Core.html#create/4-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.create/4","doc":"- `{:ok, Ecto.Schema.t()}` - On success\n- `{:error, Ecto.Changeset.t()}` - On validation failure","ref":"Aurora.Ctx.Core.html#create/4-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.create/4","doc":"alias MyApp.{Repo, Product}\n    Core.create(Repo, Product, %{name: \"Widget\"})\n    #=> {:ok, %Product{name: \"Widget\"}}\n\n    Core.create(Repo, Product, :custom_changeset, %{\n      name: \"Widget\",\n      status: \"active\"\n    })\n    #=> {:ok, %Product{name: \"Widget\", status: \"active\"}}","ref":"Aurora.Ctx.Core.html#create/4-examples"},{"type":"function","title":"Aurora.Ctx.Core.create!/3","doc":"Creates a new record, raising on errors.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to create\n- attrs (map) - Attributes for the new record\n\nReturns created schema or raises on error.","ref":"Aurora.Ctx.Core.html#create!/3"},{"type":"function","title":"Aurora.Ctx.Core.create!/4","doc":"Creates a new record, raising on errors.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to create\n- changeset_function (atom) - Changeset function to use\n- attrs (map) - Attributes for the new record\n\nReturns created schema or raises on error.","ref":"Aurora.Ctx.Core.html#create!/4"},{"type":"function","title":"Aurora.Ctx.Core.delete/2","doc":"Deletes the given entity.\n\nParameters:\n- repo_module (module): The Ecto.Repo module to use\n- entity (Ecto.Schema.t()): The entity to delete\n\nReturns: {:ok, Ecto.Schema.t()} | {:error, Ecto.Changeset.t()}","ref":"Aurora.Ctx.Core.html#delete/2"},{"type":"function","title":"Aurora.Ctx.Core.delete!/2","doc":"Deletes the given entity, raising on errors.\n\nParameters:\n- repo_module (module): The Ecto.Repo module to use\n- entity (Ecto.Schema.t()): The entity to delete\n\nReturns: Ecto.Schema.t()\nRaises: Ecto.StaleEntryError","ref":"Aurora.Ctx.Core.html#delete!/2"},{"type":"function","title":"Aurora.Ctx.Core.exclude_clauses/2","doc":"Excludes query clauses from an Ecto query.\n\nParameters:\n- query (Ecto.Query.t()) - Query to modify\n- clauses (atom | [atom]) - Clause(s) to exclude\n\nReturns modified query.","ref":"Aurora.Ctx.Core.html#exclude_clauses/2"},{"type":"function","title":"Aurora.Ctx.Core.get/4","doc":"Gets a record by id.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to query\n- id (term) - Primary key to look up\n- opts (keyword) - Optional query parameters\n\nReturns found schema or nil if not found.","ref":"Aurora.Ctx.Core.html#get/4"},{"type":"function","title":"Aurora.Ctx.Core.get!/4","doc":"Gets a record by id, raising if not found.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to query\n- id (term) - Primary key to look up\n- opts (keyword) - Optional query parameters\n\nReturns found schema or raises Ecto.NoResultsError.","ref":"Aurora.Ctx.Core.html#get!/4"},{"type":"function","title":"Aurora.Ctx.Core.list/3","doc":"Lists all records for a schema with optional filtering and sorting.","ref":"Aurora.Ctx.Core.html#list/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.list/3","doc":"- repo_module (module()) - Ecto.Repo module to use\n- schema_module (module()) - Schema module to query\n- opts (keyword()) - Optional query parameters:\n  - `:where` - Filter conditions\n  - `:order_by` - Sort specifications\n  - `:preload` - Associations to preload\n  - `:limit` - Maximum number of records","ref":"Aurora.Ctx.Core.html#list/3-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.list/3","doc":"- list(Ecto.Schema.t())","ref":"Aurora.Ctx.Core.html#list/3-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.list/3","doc":"alias MyApp.{Repo, Product}\n    Core.list(Repo, Product)\n    #=> [%Product{}, ...]\n\n    Core.list(Repo, Product,\n      where: [status: :active],\n      order_by: [desc: :inserted_at],\n      preload: [:category]\n    )\n    #=> [%Product{category: %Category{}}, ...]","ref":"Aurora.Ctx.Core.html#list/3-examples"},{"type":"function","title":"Aurora.Ctx.Core.list_paginated/3","doc":"Lists records with pagination support.","ref":"Aurora.Ctx.Core.html#list_paginated/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.list_paginated/3","doc":"- repo_module (module()) - Ecto.Repo module to use\n- schema_module (module()) - Schema module to query\n- opts (keyword()) - Query options including:\n  - `:paginate` - Pagination settings (%{page: integer, per_page: integer})\n  - `:where` - Filter conditions\n  - `:order_by` - Sort specifications\n  - `:preload` - Associations to preload","ref":"Aurora.Ctx.Core.html#list_paginated/3-parameters"},{"type":"function","title":"Configuration - Aurora.Ctx.Core.list_paginated/3","doc":"Default pagination settings can be configured in your `config.exs`:\n\n    config :aurora_ctx, :pagination,\n      page: 1,\n      per_page: 40\n\nIf not configured, defaults to page: 1, per_page: 40.","ref":"Aurora.Ctx.Core.html#list_paginated/3-configuration"},{"type":"function","title":"Returns - Aurora.Ctx.Core.list_paginated/3","doc":"- Aurora.Ctx.Pagination.t()","ref":"Aurora.Ctx.Core.html#list_paginated/3-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.list_paginated/3","doc":"alias MyApp.{Repo, Product}\n    Core.list_paginated(Repo, Product,\n      paginate: %{page: 1, per_page: 20},\n      where: [category_id: 1],\n      order_by: [desc: :inserted_at]\n    )\n    #=> %Aurora.Ctx.Pagination{\n    #=>   entries: [%Product{}],\n    #=>   page: 1,\n    #=>   per_page: 20,\n    #=>   entries_count: 50,\n    #=>   pages_count: 3\n    #=> }","ref":"Aurora.Ctx.Core.html#list_paginated/3-examples"},{"type":"function","title":"Aurora.Ctx.Core.new/3","doc":"Initializes a new schema struct with optional attributes and preloads.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to initialize\n- attrs (map) - Initial attributes\n- opts (keyword) - Optional preload parameters\n\nReturns initialized schema struct.","ref":"Aurora.Ctx.Core.html#new/3"},{"type":"function","title":"Aurora.Ctx.Core.new/4","doc":"Initializes a new struct with optional preloads.\n\nParameters:\n- repo_module (module): The Ecto.Repo module to use\n- schema_module (module): The Ecto.Schema module to initialize\n- opts (keyword): Optional preload parameters\n\nReturns: Ecto.Schema.t()","ref":"Aurora.Ctx.Core.html#new/4"},{"type":"function","title":"Aurora.Ctx.Core.next_page/1","doc":"Moves to next page in paginated results.\n\nParameters:\n- paginate (Pagination.t()) - Current pagination state\n\nReturns updated Pagination struct with next page entries.","ref":"Aurora.Ctx.Core.html#next_page/1"},{"type":"function","title":"Aurora.Ctx.Core.previous_page/1","doc":"Moves to previous page in paginated results.\n\nParameters:\n- paginate (Pagination.t()) - Current pagination state\n\nReturns updated Pagination struct with previous page entries.","ref":"Aurora.Ctx.Core.html#previous_page/1"},{"type":"function","title":"Aurora.Ctx.Core.to_page/2","doc":"Changes to a specific page in paginated results.\n\nParameters:\n- paginate (Pagination.t()) - Current pagination state\n- page (integer) - Target page number\n\nReturns updated Pagination struct with new page entries.","ref":"Aurora.Ctx.Core.html#to_page/2"},{"type":"function","title":"Aurora.Ctx.Core.update/3","doc":"Updates an entity with given attributes.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- entity (Ecto.Schema.t()) - Entity to update\n- attrs (map) - Update attributes\n\nReturns {:ok, schema} on success, {:error, changeset} on failure.","ref":"Aurora.Ctx.Core.html#update/3"},{"type":"function","title":"Aurora.Ctx.Core.update/4","doc":"Updates an entity using a specific changeset function.","ref":"Aurora.Ctx.Core.html#update/4"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.update/4","doc":"- repo_module (module()) - Ecto.Repo module to use\n- entity (Ecto.Schema.t()) - Existing record to update\n- changeset_function (atom()) - Custom changeset function\n- attrs (map()) - Update attributes","ref":"Aurora.Ctx.Core.html#update/4-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.update/4","doc":"- `{:ok, Ecto.Schema.t()}` - On success\n- `{:error, Ecto.Changeset.t()}` - On validation failure","ref":"Aurora.Ctx.Core.html#update/4-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.update/4","doc":"alias MyApp.{Repo, Product}\n    product = %Product{name: \"Old Name\"}\n    Core.update(Repo, product, %{name: \"New Name\"})\n    #=> {:ok, %Product{name: \"New Name\"}}","ref":"Aurora.Ctx.Core.html#update/4-examples"},{"type":"module","title":"Aurora.Ctx.Pagination","doc":"Struct for handling pagination state.","ref":"Aurora.Ctx.Pagination.html"},{"type":"module","title":"Configuration - Aurora.Ctx.Pagination","doc":"Default pagination settings can be configured in your `config.exs`:\n\n    config :aurora_ctx, :pagination,\n      page: 1,\n      per_page: 40\n\nIf not configured, defaults to page: 1, per_page: 40.","ref":"Aurora.Ctx.Pagination.html#module-configuration"},{"type":"function","title":"Aurora.Ctx.Pagination.new/0","doc":"Creates a new pagination struct with safe defaults.\n\nReturns:\n  t()","ref":"Aurora.Ctx.Pagination.html#new/0"},{"type":"function","title":"Aurora.Ctx.Pagination.new/1","doc":"Creates a new pagination struct with safe defaults.\n\nParameters:\n  - attrs (map | keyword): Pagination attributes\n    - :page (pos_integer): Page number (default: 1)\n    - :per_page (pos_integer): Items per page (default: 40)\n\nReturns:\n  t()","ref":"Aurora.Ctx.Pagination.html#new/1"},{"type":"type","title":"Aurora.Ctx.Pagination.t/0","doc":"","ref":"Aurora.Ctx.Pagination.html#t:t/0"},{"type":"task","title":"mix test.setup","doc":"A Mix task for setting up the test environment by creating and migrating the test database.","ref":"Mix.Tasks.Test.Setup.html"},{"type":"function","title":"Mix.Tasks.Test.Setup.run/1","doc":"Runs the test setup task by creating and migrating the test database.\n\nParameters:\n- args: list | nil - Command line arguments (not used)\n\nReturns:\n- any","ref":"Mix.Tasks.Test.Setup.html#run/1"},{"type":"module","title":"Aurora.Ctx","doc":"A module for automatically generating CRUD operations for Ecto schemas.\n\nThis module provides macros to generate standard database operation functions\nbased on Ecto schemas. It simplifies the creation of context modules by\nautomatically implementing common CRUD operations.","ref":"Aurora.Ctx.html"},{"type":"module","title":"Usage - Aurora.Ctx","doc":"defmodule MyApp.Accounts do\n      use Aurora.Ctx\n\n      ctx_register_schema(User)\n      # or with options\n      ctx_register_schema(User, CustomRepo,\n        changeset_function: :custom_changeset,\n        create_changeset_function: :create_changeset\n      )\n    end","ref":"Aurora.Ctx.html#module-usage"},{"type":"module","title":"Generated Functions - Aurora.Ctx","doc":"For a schema with source name \"users\" and module name \"user\", the following functions are generated:","ref":"Aurora.Ctx.html#module-generated-functions"},{"type":"module","title":"List Functions - Aurora.Ctx","doc":"- `list_users/0` - List all records\n- `list_users/1` - List all records with options\n- `list_users_paginated/0` - List records with pagination\n- `list_users_paginated/1` - List records with pagination and options\n- `count_users/0` - Count total records\n- `count_users/1` - Count records with options","ref":"Aurora.Ctx.html#module-list-functions"},{"type":"module","title":"Create Functions - Aurora.Ctx","doc":"- `create_user/0` - Create a record with empty attributes\n- `create_user/1` - Create a record with given attributes\n- `create_user!/0` - Create a record with empty attributes (raises on error)\n- `create_user!/1` - Create a record with given attributes (raises on error)","ref":"Aurora.Ctx.html#module-create-functions"},{"type":"module","title":"Get Functions - Aurora.Ctx","doc":"- `get_user/1` - Get a record by ID\n- `get_user/2` - Get a record by ID with options\n- `get_user!/1` - Get a record by ID (raises if not found)\n- `get_user!/2` - Get a record by ID with options (raises if not found)","ref":"Aurora.Ctx.html#module-get-functions"},{"type":"module","title":"Delete Functions - Aurora.Ctx","doc":"- `delete_user/1` - Delete a record\n- `delete_user!/1` - Delete a record (raises on error)","ref":"Aurora.Ctx.html#module-delete-functions"},{"type":"module","title":"Change Functions - Aurora.Ctx","doc":"- `change_user/1` - Create a changeset from a record\n- `change_user/2` - Create a changeset from a record with attributes","ref":"Aurora.Ctx.html#module-change-functions"},{"type":"module","title":"Update Functions - Aurora.Ctx","doc":"- `update_user/1` - Update a record\n- `update_user/2` - Update a record with attributes","ref":"Aurora.Ctx.html#module-update-functions"},{"type":"module","title":"New Functions - Aurora.Ctx","doc":"- `new_user/0` - Initialize a new struct\n- `new_user/1` - Initialize a new struct with attributes\n- `new_user/2` - Initialize a new struct with attributes and options","ref":"Aurora.Ctx.html#module-new-functions"},{"type":"macro","title":"Aurora.Ctx.ctx_register_schema/3","doc":"Registers a schema for CRUD function generation.","ref":"Aurora.Ctx.html#ctx_register_schema/3"},{"type":"macro","title":"Parameters - Aurora.Ctx.ctx_register_schema/3","doc":"* `schema_module` (module()) - The Ecto schema module to generate functions for\n  * `repo` (module() | nil) - (Optional) The Ecto.Repo to use for database operations. Can be set globally with `@ctx_repo_module`\n  * `opts` (keyword()) - (Optional) Configuration options:\n    * `:changeset_function` (atom()) - The function to use for changesets (default: `:changeset`)\n    * `:create_changeset_function` (atom()) - A specific function to use for creation changesets","ref":"Aurora.Ctx.html#ctx_register_schema/3-parameters"},{"type":"macro","title":"Repository Configuration - Aurora.Ctx.ctx_register_schema/3","doc":"The Ecto.Repo can be configured in two ways:\n  1. Passing it as the second argument to `ctx_register_schema/3`\n  2. Setting the `@ctx_repo_module` module attribute in your context\n\nIf neither is specified, it will attempt to use `YourApp.Repo` based on your\ncontext module's namespace.","ref":"Aurora.Ctx.html#ctx_register_schema/3-repository-configuration"},{"type":"macro","title":"Example - Aurora.Ctx.ctx_register_schema/3","doc":"# Using explicit repo\n    ctx_register_schema(User, MyApp.Repo)\n\n    # Using module attribute\n    @ctx_repo_module MyApp.Repo\n    ctx_register_schema(User)\n\n    # With additional options\n    ctx_register_schema(User, MyApp.Repo, changeset_function: :custom_changeset)","ref":"Aurora.Ctx.html#ctx_register_schema/3-example"},{"type":"function","title":"Aurora.Ctx.implementable_functions/1","doc":"Lists all implementable CRUD functions for a given schema.\n\nThis function is used internally to determine which functions should be generated\nfor a given schema. It returns a list of maps containing function definitions.","ref":"Aurora.Ctx.html#implementable_functions/1"},{"type":"function","title":"Parameters - Aurora.Ctx.implementable_functions/1","doc":"* `schema_module` - The Ecto schema module to analyze","ref":"Aurora.Ctx.html#implementable_functions/1-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.implementable_functions/1","doc":"* List of maps containing:\n    * `:type` - The function type (`:list`, `:create`, etc.)\n    * `:name` - The function name\n    * `:arity` - The function arity","ref":"Aurora.Ctx.html#implementable_functions/1-returns"},{"type":"module","title":"Aurora.Ctx.QueryBuilder","doc":"Provides functions for building and composing Ecto queries with common operations like filtering,\nsorting, pagination, and preloading associations.","ref":"Aurora.Ctx.QueryBuilder.html"},{"type":"function","title":"Aurora.Ctx.QueryBuilder.options/2","doc":"Applies a list of query options to an Ecto query.","ref":"Aurora.Ctx.QueryBuilder.html#options/2"},{"type":"function","title":"Parameters - Aurora.Ctx.QueryBuilder.options/2","doc":"- query (Ecto.Query.t() | nil) - Core query to modify\n- options (keyword) - List of query options to apply","ref":"Aurora.Ctx.QueryBuilder.html#options/2-parameters"},{"type":"function","title":"Options - Aurora.Ctx.QueryBuilder.options/2","doc":"- :preload (atom | list) - Associations to preload\n- :where (keyword | map | tuple) - Filter conditions\n  - Simple equality: `{:field, value}`\n  - Comparison: `{:field, operator, value}` where operator can be:\n    - :greater_than, :gt - Greater than\n    - :greater_equal_than, :ge - Greater than or equal\n    - :less_than, :lt - Less than\n    - :less_equal_than, :le - Less than or equal\n    - :equal_to, :eq - Equal to\n  - Range: `{:field, :between, start_value, end_value}`\n- :or_where (keyword | map | tuple) - Same as :where but combines with OR\n- :paginate (map) - Pagination options with keys:\n  - :page (integer) - Page number\n  - :per_page (integer) - Items per page\n- :sort (atom | tuple | list) - Sorting options:\n  - field (atom | {:asc | :desc, field}) - Field to sort by ascending\n  - fields ([{:asc | :desc, field}]) - List of sort fields specifications","ref":"Aurora.Ctx.QueryBuilder.html#options/2-options"},{"type":"function","title":"Returns - Aurora.Ctx.QueryBuilder.options/2","doc":"- Ecto.Query.t() - Modified query with all options applied","ref":"Aurora.Ctx.QueryBuilder.html#options/2-returns"},{"type":"function","title":"Examples - Aurora.Ctx.QueryBuilder.options/2","doc":"query = from(p in Product)\n    QueryBuilder.options(query,\n      where: [status: :active],\n      preload: [:category],\n      sort: [desc: :inserted_at],\n      paginate: %{page: 1, per_page: 20}\n    )","ref":"Aurora.Ctx.QueryBuilder.html#options/2-examples"},{"type":"extras","title":"Changelog for v0.x","doc":"# Changelog for v0.x","ref":"changelog.html"},{"type":"extras","title":"v0.1.0 - Changelog for v0.x","doc":"Requires Elixir v1.17+\nCompatible with Ecto 3.2+","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Features - Changelog for v0.x","doc":"- Automatic CRUD function generation for Ecto schemas\n- Customizable repository module selection\n- Customizable changeset function names\n- Support for both safe and raising versions of operations\n- Built-in support for preloading associations","ref":"changelog.html#features"},{"type":"extras","title":"Overview","doc":"# Overview\n\nAurora.Ctx is a code generation library that implements the Context pattern in Phoenix applications. It automatically generates standardized database interaction functions following Phoenix's architectural guidelines, while maintaining full compatibility with Ecto's query composition and changesets. The library supports both standard CRUD operations and advanced query features like filtering, sorting, and pagination.","ref":"overview.html"},{"type":"extras","title":"Key Benefits - Overview","doc":"- **Reduced Boilerplate**: Automatically generates common database access functions\n- **Consistent API**: Ensures uniform function names and patterns across contexts\n- **Type Safety**: All generated functions include proper typespecs\n- **Flexible**: Supports customization through options and hooks\n- **Maintainable**: Less code to maintain, centralized logic","ref":"overview.html#key-benefits"},{"type":"extras","title":"Getting Started - Overview","doc":"Add `aurora_ctx` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:aurora_ctx, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"overview.html#getting-started"},{"type":"extras","title":"Basic Usage - Overview","doc":"","ref":"overview.html#basic-usage"},{"type":"extras","title":"Creating an Ecto Schema - Overview","doc":"> **Tip**: Consider using [TypedEctoSchema](https://hexdocs.pm/typed_ecto_schema) to reduce boilerplate in your schema definitions and add compile-time type checking.\n\nLet's start with a typical Ecto schema definition that we'll use throughout the examples:\n\n```elixir\ndefmodule MyApp.Inventory.Product do\n  use Ecto.Schema\n  \n  schema \"products\" do\n    field :name, :string\n    field :price, :decimal\n    \n    timestamps()\n  end\n\n  def changeset(product, attrs) do\n    product\n    |> cast(attrs, [:name, :price])\n    |> validate_required([:name, :price])\n  end\nend\n```","ref":"overview.html#creating-an-ecto-schema"},{"type":"extras","title":"Traditional Context Approach - Overview","doc":"Without Aurora.Ctx, you would typically write a context module with all the CRUD operations manually:\n\n```elixir\ndefmodule MyApp.Inventory do\n  import Ecto.Query\n  alias MyApp.Repo\n  alias MyApp.Inventory.Product\n\n  def list_products do\n    Product |> from() |> Repo.all()\n  end\n\n  def get_product(id) do\n    Product |> Repo.get(id)\n  end\n\n  def get_product!(id) do\n    Product |> Repo.get!(id)\n  end\n\n  def create_product(attrs) do\n    %Product{}\n    |> Product.changeset(attrs)\n    |> Repo.insert()\n  end\n\n  def update_product(%Product{} = product, attrs) do\n    product\n    |> Product.changeset(attrs)\n    |> Repo.update()\n  end\n\n  def delete_product(%Product{} = product) do\n    Repo.delete(product)\n  end\n\n  def change_product(%Product{} = product, attrs \\\\ %{}) do\n    Product.changeset(product, attrs)\n  end\nend\n```","ref":"overview.html#traditional-context-approach"},{"type":"extras","title":"Context Generation with Aurora.Ctx - Overview","doc":"With Aurora.Ctx, the same functionality can be achieved with just a few lines of code:\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n\n  # The repository will default to MyApp.Repo  \n  ctx_register_schema(MyApp.Inventory.Product)\nend\n```\n\nNow you can use the context just like the traditional approach:\n\n```elixir\n# List all products\nproducts = Inventory.list_products()\n\n# Get a specific product\nproduct = Inventory.get_product!(1)\n\n# Create a new product\n{:ok, product} = Inventory.create_product(%{name: \"Widget\", price: 19.99})\n\n# Update a product\n{:ok, product} = Inventory.update_product(product, %{price: 29.99})\n\n# Delete a product\n{:ok, _deleted} = Inventory.delete_product(product)\n```","ref":"overview.html#context-generation-with-aurora-ctx"},{"type":"extras","title":"Function Customization - Overview","doc":"You can override any generated function by defining it before calling `ctx_register_schema`:\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n\n  def list_products do\n    # Custom implementation with specific business logic\n    MyApp.Inventory.Product\n    |> where([p], p.active == true)\n    |> order_by([p], [desc: p.inserted_at])\n    |> MyApp.Repo.all()\n  end\n\n  ctx_register_schema(MyApp.Inventory.Product)\nend\n```","ref":"overview.html#function-customization"},{"type":"extras","title":"Generated Functions - Overview","doc":"For each schema, the following functions are automatically generated:","ref":"overview.html#generated-functions"},{"type":"extras","title":"List Functions - Overview","doc":"- `list_*()`- List all records\n- `list_*(opts)` - List all records with options\n- `list_*_paginated()` - List records with pagination\n- `list_*_paginated(opts)` - List records with pagination and options\n- `count_*()` - Count total records\n- `count_*(opts)` - Count records with options","ref":"overview.html#list-functions"},{"type":"extras","title":"Create Functions - Overview","doc":"- `create_*()` - Create a record with empty attributes\n- `create_*(attrs)` - Create a record with given attributes\n- `create_*!()` - Create a record with empty attributes (raises on error)\n- `create_*!(attrs)` - Create a record with given attributes (raises on error)","ref":"overview.html#create-functions"},{"type":"extras","title":"Get Functions - Overview","doc":"- `get_*(id)` - Get a record by ID\n- `get_*(id, opts)` - Get a record by ID with options\n- `get_*!(id)` - Get a record by ID (raises if not found)\n- `get_*!(id, opts)` - Get a record by ID with options (raises if not found)","ref":"overview.html#get-functions"},{"type":"extras","title":"Delete Functions - Overview","doc":"- `delete_*(entity)` - Delete a record\n- `delete_*!(entity)` - Delete a record (raises on error)","ref":"overview.html#delete-functions"},{"type":"extras","title":"Change Functions - Overview","doc":"- `change_*(entity)` - Create a changeset from a record\n- `change_*(entity, attrs)` - Create a changeset from a record with attributes","ref":"overview.html#change-functions"},{"type":"extras","title":"Update Functions - Overview","doc":"- `update_*(entity)` - Update a record\n- `update_*(entity, attrs)` - Update a record with attributes","ref":"overview.html#update-functions"},{"type":"extras","title":"New Functions - Overview","doc":"- `new_*()` - Initialize a new struct\n- `new_*(attrs)` - Initialize a new struct with attributes\n- `new_*(attrs, opts)` - Initialize a new struct with attributes and options\n\nCheck the [Functions](functions.html) guide for detailed documentation of each function.","ref":"overview.html#new-functions"},{"type":"extras","title":"Direct Core Usage - Overview","doc":"Sometimes you may want to use the core operations directly without generating context functions. This approach is particularly useful in scenarios such as:\n\n- When working with dynamic schemas determined at runtime\n- Building generic admin interfaces or APIs that handle multiple schemas\n- Creating reusable modules that work with any schema\n- Testing and prototyping before defining your final context structure\n\nHere's how to use the core module directly:\n\n```elixir\nalias Aurora.Ctx.Core\n\n# List all records\nproducts = Core.list(MyApp.Repo, MyApp.Inventory.Product)\n\n# List with filtering and sorting\nproducts = Core.list(MyApp.Repo, MyApp.Inventory.Product,\n  where: [status: :active],\n  order_by: [desc: :inserted_at],\n  preload: [:category]\n)\n\n# Paginated listing\npage = Core.list_paginated(MyApp.Repo, MyApp.Inventory.Product,\n  paginate: %{page: 1, per_page: 20}\n)\n\n# Create a record\n{:ok, product} = Core.create(MyApp.Repo, MyApp.Inventory.Product, %{\n  name: \"Widget\",\n  price: 19.99\n})\n\n# Update a record\n{:ok, product} = Core.update(MyApp.Repo, product, %{price: 29.99})\n\n# Delete a record\n{:ok, _deleted} = Core.delete(MyApp.Repo, product)\n```","ref":"overview.html#direct-core-usage"},{"type":"extras","title":"Context vs Direct Core Usage - Overview","doc":"While using the Core module directly provides flexibility, using proper contexts has several advantages:\n\n1. **Encapsulation**: Contexts hide implementation details and provide a cleaner API\n2. **Business Logic**: Contexts are the ideal place to add business rules and validations\n3. **Consistency**: Generated functions ensure consistent naming and behavior across your app\n4. **Documentation**: Function names in contexts are more descriptive of their business purpose\n5. **Maintainability**: Changes to database operations can be centralized in contexts\n6. **Type Safety**: Generated functions include proper typespecs for better tooling support\n\nConsider using proper contexts for most application code, and reserve direct Core usage for special cases where flexibility is more important than the benefits listed above.","ref":"overview.html#context-vs-direct-core-usage"},{"type":"extras","title":"Generated Functions Reference","doc":"# Generated Functions Reference\n\nThis guide details all the functions that Aurora.Ctx generates for your context modules.","ref":"functions.html"},{"type":"extras","title":"List Functions - Generated Functions Reference","doc":"```elixir\nlist_products()                # Returns [%Product{}]\nlist_products(opts)            # Returns [%Product{}] with filtering/sorting\nlist_products_paginated()      # Returns %Pagination{} with default options\nlist_products_paginated(opts)  # Returns %Pagination{} with custom options\ncount_products()               # Returns total count of records\ncount_products(opts)           # Returns filtered count of records\n```","ref":"functions.html#list-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Get active products sorted by price\nproducts = list_products(\n  where: [status: :active],\n  order_by: [asc: :price]\n)\n```\n\nThe `opts` parameter supports:\n- `:preload` - Associations to preload\n- `:where` - Filter conditions (equality, comparison, range)\n- `:or_where` - Alternative filter conditions combined with OR\n- `:order_by` - Sorting specification\n- `:paginate` - Pagination options (page, per_page)","ref":"functions.html#example"},{"type":"extras","title":"Get Functions - Generated Functions Reference","doc":"```elixir\nget_product(id)                # Returns %Product{} or nil\nget_product(id, opts)          # Returns %Product{} or nil with preloads\nget_product!(id)               # Returns %Product{} or raises Ecto.NoResultsError\nget_product!(id, opts)         # Returns %Product{} or raises, with preloads\n```","ref":"functions.html#get-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Get product with associated category\nproduct = get_product!(1, preload: [:category])\n```","ref":"functions.html#example"},{"type":"extras","title":"Create Functions - Generated Functions Reference","doc":"```elixir\ncreate_product()               # Returns {:ok, %Product{}} with defaults\ncreate_product(attrs)          # Returns {:ok, %Product{}} or {:error, changeset}\ncreate_product!()              # Returns %Product{} or raises\ncreate_product!(attrs)         # Returns %Product{} or raises\n```","ref":"functions.html#create-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n{:ok, product} = create_product(%{\n  name: \"Widget Pro\",\n  price: Decimal.new(\"29.99\")\n})\n```","ref":"functions.html#example"},{"type":"extras","title":"Update Functions - Generated Functions Reference","doc":"```elixir\nupdate_product(entity)         # Returns {:ok, %Product{}} with no changes\nupdate_product(entity, attrs)  # Returns {:ok, %Product{}} or {:error, changeset}\n```","ref":"functions.html#update-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n{:ok, updated} = update_product(product, %{price: Decimal.new(\"39.99\")})\n```","ref":"functions.html#example"},{"type":"extras","title":"Delete Functions - Generated Functions Reference","doc":"```elixir\ndelete_product(entity)         # Returns {:ok, %Product{}} or {:error, changeset}\ndelete_product!(entity)        # Returns %Product{} or raises\n```","ref":"functions.html#delete-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n{:ok, deleted} = delete_product(product)\n```","ref":"functions.html#example"},{"type":"extras","title":"Change Functions - Generated Functions Reference","doc":"```elixir\nchange_product(entity)         # Returns %Ecto.Changeset{} with no changes\nchange_product(entity, attrs)  # Returns %Ecto.Changeset{} with changes\n```","ref":"functions.html#change-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\nchangeset = change_product(product, %{price: Decimal.new(\"49.99\")})\n```","ref":"functions.html#example"},{"type":"extras","title":"New Functions - Generated Functions Reference","doc":"```elixir\nnew_product()                  # Returns %Product{} struct\nnew_product(attrs)             # Returns %Product{} with attributes\nnew_product(attrs, opts)       # Returns %Product{} with attributes and preloads\n```","ref":"functions.html#new-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\nproduct = new_product(%{name: \"Widget Pro\", price: Decimal.new(\"29.99\")})\n```","ref":"functions.html#example"},{"type":"extras","title":"Query Options - Generated Functions Reference","doc":"","ref":"functions.html#query-options"},{"type":"extras","title":"Where Conditions - Generated Functions Reference","doc":"```elixir\n# Simple equality\nwhere: [status: :active]\n\n# Comparisons\nwhere: {:price, :greater_than, 100}\nwhere: {:price, :less_than, 200}\nwhere: {:date, :greater_equal_than, ~D[2023-01-01]}\n\n# Ranges\nwhere: {:price, :between, 100, 200}\n\n# Multiple conditions\nwhere: [status: :active, price: {:greater_than, 100}]\n```","ref":"functions.html#where-conditions"},{"type":"extras","title":"Sorting - Generated Functions Reference","doc":"```elixir\n# Simple sort\norder_by: :inserted_at           # ascending\norder_by: {:desc, :inserted_at}  # descending\n\n# Multiple fields\norder_by: [{:desc, :inserted_at}, {:asc, :name}]\n```","ref":"functions.html#sorting"},{"type":"extras","title":"Pagination - Generated Functions Reference","doc":"```elixir\n# Default pagination\npaginate: %{page: 1, per_page: 20}\n```\n\nFor implementation examples, check out the [Examples](examples.html) guide.","ref":"functions.html#pagination"},{"type":"extras","title":"Examples","doc":"# Examples\n\nExamples of using Aurora.Ctx in common scenarios.","ref":"examples.html"},{"type":"extras","title":"Basic CRUD Operations - Examples","doc":"Basic context setup and standard database operations.\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx,\n    schema_module: MyApp.Inventory.Product,\n    repo_module: MyApp.Repo\nend\n\n# Usage examples\nalias MyApp.Inventory\n\n# Create a product\n{:ok, product} = Inventory.create_product(%{\n  name: \"Premium Widget\",\n  price: Decimal.new(\"29.99\")\n})\n\n# List products with sorting\nproducts = Inventory.list_products(order_by: [desc: :inserted_at])\n\n# Get product with preloaded associations\n{:ok, product} = Inventory.get_product(1, preload: [:category, :supplier])\n\n# Update a product\n{:ok, updated} = Inventory.update_product(product, %{price: Decimal.new(\"39.99\")})\n\n# Delete a product\n{:ok, _deleted} = Inventory.delete_product(product)\n```","ref":"examples.html#basic-crud-operations"},{"type":"extras","title":"Query Usage - Examples","doc":"More complex database operations using query options.","ref":"examples.html#query-usage"},{"type":"extras","title":"Custom Query Options - Examples","doc":"Combining filtering, preloading, and sorting in queries.\n\n```elixir\n# List active products with preloaded categories\nproducts = Inventory.list_products(\n  where: [status: \"active\"],\n  preload: [:category],\n  order_by: [asc: :name],\n  limit: 10\n)\n\n# Get a product with specific preloads\nproduct = Inventory.get_product!(1, \n  preload: [\n    category: [:parent_category],\n    reviews: [:user]\n  ]\n)\n```","ref":"examples.html#custom-query-options"},{"type":"extras","title":"Working with Changesets - Examples","doc":"Data validation and form handling with changesets.\n\n```elixir\n# Create a changeset for a new product\nchangeset = Inventory.change_product(%Product{})\n\n# Create a changeset with attributes\nchangeset = Inventory.change_product(%Product{}, %{\n  name: \"New Widget\",\n  price: \"19.99\"\n})\n\n# Working with forms in Phoenix\ndef new(conn, _params) do\n  changeset = Inventory.change_product(%Product{})\n  render(conn, \"new.html\", changeset: changeset)\nend\n\ndef create(conn, %{\"product\" => product_params}) do\n  case Inventory.create_product(product_params) do\n    {:ok, product} -> \n      redirect(to: Routes.product_path(conn, :show, product))\n    {:error, changeset} ->\n      render(conn, \"new.html\", changeset: changeset)\n  end\nend\n```","ref":"examples.html#working-with-changesets"},{"type":"extras","title":"Query Options - Examples","doc":"Available options for customizing database queries.","ref":"examples.html#query-options"},{"type":"extras","title":"Filtering - Examples","doc":"Filter query results using different conditions.\n\n```elixir\n# Multiple conditions\nproducts = Inventory.list_products(\n  where: [\n    status: \"active\",\n    category_id: 1\n  ]\n)\n\n# Comparison operators\nproducts = Inventory.list_products(\n  where: [\n    {:price, :gt, Decimal.new(\"100.00\")},\n    {:stock_level, :le, 5}\n  ]\n)\n\n# Range queries\nproducts = Inventory.list_products(\n  where: {:price, :between, Decimal.new(\"10.00\"), Decimal.new(\"50.00\")}\n)\n\n# OR conditions\nproducts = Inventory.list_products(\n  where: [status: \"active\"],\n  or_where: [status: \"pending_review\"]\n)\n```","ref":"examples.html#filtering"},{"type":"extras","title":"Sorting - Examples","doc":"Order query results with various sorting strategies.\n\n```elixir\n# Multiple sort fields\nproducts = Inventory.list_products(\n  order_by: [\n    asc: :category_id,\n    desc: :price,\n    asc: :name\n  ]\n)\n\n# Null handling\nproducts = Inventory.list_products(\n  order_by: [\n    asc_nulls_last: :discontinued_at,\n    desc_nulls_first: :updated_at\n  ]\n)\n```","ref":"examples.html#sorting"},{"type":"extras","title":"Controlled Pagination - Examples","doc":"Implement and navigate through paginated results. Results are wrapped in an `Aurora.Ctx.Pagination` struct that provides additional metadata and navigation helpers.\n\n```elixir\n# Basic pagination\npage1 = Inventory.list_products_paginated(\n  paginate: %{page: 1, per_page: 20}\n)\n\n# Access pagination info\nIO.puts \"Total entries: #{page1.entries_count}\"\nIO.puts \"Total pages: #{page1.pages_count}\"\nIO.puts \"Current page: #{page1.page}\"\n\n# Navigate through pages\nnext_page = Aurora.Ctx.Core.next_page(page1)\nprev_page = Aurora.Ctx.Core.previous_page(next_page)\npage5 = Aurora.Ctx.Core.to_page(page1, 5)\n\n# Combine with other options\nfiltered_page = Inventory.list_products_paginated(\n  where: [category_id: 1],\n  order_by: [desc: :inserted_at],\n  paginate: %{page: 1, per_page: 10}\n)\n```","ref":"examples.html#controlled-pagination"},{"type":"extras","title":"Preloading Associates - Examples","doc":"Load associated data with simple and nested preloads.\n\n```elixir\n# Nested preloads\nproduct = Inventory.get_product!(1,\n  preload: [\n    category: [:parent_category, :subcategories],\n    reviews: [:user, comments: [:user]],\n    supplier: [:address, :contacts]\n  ]\n)\n\n# Filtered preloads with custom queries\nproducts = Inventory.list_products(\n  preload: [\n    reviews: from(r in Review, where: r.rating > 3)\n  ]\n)\n```","ref":"examples.html#preloading-associates"},{"type":"extras","title":"Working with Transactions - Examples","doc":"Combine multiple database operations in transactions.\n\n```elixir\nalias Ecto.Multi\nalias MyApp.{Inventory, Orders}\n\n# Complex multi-operation transaction\ndef create_order_with_inventory_update(attrs) do\n  Multi.new()\n  |> Multi.run(:product, fn repo, _ ->\n    Inventory.get_product(attrs.product_id)\n  end)\n  |> Multi.run(:check_stock, fn _repo, %{product: product} ->\n    if product.stock >= attrs.quantity do\n      {:ok, product}\n    else\n      {:error, :insufficient_stock}\n    end\n  end)\n  |> Multi.run(:order, fn repo, %{product: product} ->\n    Orders.create_order(attrs)\n  end)\n  |> Multi.run(:update_inventory, fn repo, %{product: product, order: order} ->\n    Inventory.update_product(product, %{\n      stock: product.stock - order.quantity\n    })\n  end)\n  |> MyApp.Repo.transaction()\nend\n\n# Usage\ncase create_order_with_inventory_update(%{product_id: 1, quantity: 5}) do\n  {:ok, %{order: order, product: product}} ->\n    {:ok, order}\n  {:error, failed_operation, failed_value, changes_so_far} ->\n    {:error, :order_creation_failed}\nend\n```","ref":"examples.html#working-with-transactions"},{"type":"extras","title":"Custom Changesets - Examples","doc":"Configure and use custom changeset functions.\n\n```elixir\n# Using custom changeset functions\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx,\n    schema_module: MyApp.Inventory.Product,\n    repo_module: MyApp.Repo,\n    changeset_function: :custom_changeset,\n    create_changeset_function: :create_changeset\n\n  # Your additional context functions...\nend\n\n# Schema with multiple changeset functions\ndefmodule MyApp.Inventory.Product do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  schema \"products\" do\n    field :name, :string\n    field :price, :decimal\n    field :status, :string\n    timestamps()\n  end\n\n  def create_changeset(product, attrs) do\n    product\n    |> cast(attrs, [:name, :price])\n    |> validate_required([:name, :price])\n    |> validate_number(:price, greater_than: 0)\n  end\n\n  def custom_changeset(product, attrs) do\n    product\n    |> cast(attrs, [:name, :price, :status])\n    |> validate_required([:status])\n    |> validate_inclusion(:status, [\"active\", \"inactive\"])\n  end\nend\n\n# Usage\nproduct = Inventory.new_product()\nchangeset = Inventory.change_product(product, %{name: \"New Product\"})\n```\n\nFor more examples and use cases, check the test files in the [GitHub repository](https://github.com/wadvanced/aurora_ctx).","ref":"examples.html#custom-changesets"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}